/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.base.junit.shell;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;

import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.ConnectionCommand;
import org.ow2.petals.cli.api.command.DisconnectionCommand;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.api.shell.exception.ConnectionCommandAlreadyRegisteredException;
import org.ow2.petals.cli.api.shell.exception.DisconnectionCommandAlreadyRegisteredException;
import org.ow2.petals.cli.api.shell.exception.DuplicatedCommandException;

/**
 * <p>
 * A simple {@link Shell} based on {@link DummyShell}:
 * </p>
 * <ul>
 * <li>providing output stream for a standard output and an error output as {@link String},</li>
 * <li>managing exit code</li>
 * <li>managing a connection</li>
 * </ul>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class StringStreamShell extends DummyShell {

    /**
     * The real stream containing the shell output stream
     */
    private final ByteArrayOutputStream baosOutputStream = new ByteArrayOutputStream();

    /**
     * The real stream containing the shell error stream
     */
    private final ByteArrayOutputStream baosErrorStream = new ByteArrayOutputStream();

    /**
     * The output stream required by {@link Shell}
     */
    private final PrintStream printStream = new PrintStream(this.baosOutputStream);

    /**
     * The error stream required by {@link Shell}
     */
    private final PrintStream errorStream = new PrintStream(this.baosErrorStream);

    /**
     * Connection state
     */
    private boolean isConnectionEstablished = false;

    private final Preferences preferences;

    private final Map<String, Command> commands = new HashMap<>();

    private ConnectionCommand connectionCommand = null;

    private DisconnectionCommand disconnectionCommand = null;

    public StringStreamShell() {
        this(null);
    }

    public StringStreamShell(final Preferences preferences) {
        this.preferences = preferences;
    }

    @Override
    public final Map<String, Command> getCommands() {
        return this.commands;
    }

    @Override
    public void registersCommand(final Command command) throws DuplicatedCommandException, IllegalArgumentException {
        assert command != null;

        final String name = command.getName();
        if (this.commands.containsKey(name)) {
            throw new DuplicatedCommandException(name);
        } else {
            this.commands.put(name, command);
        }
    }

    @Override
    public ConnectionCommand getConnectionCommand() {
        return this.connectionCommand;
    }

    @Override
    public DisconnectionCommand getDisconnectionCommand() {
        return this.disconnectionCommand;
    }

    @Override
    public void registersConnectionCommand(final ConnectionCommand connectionCommand)
            throws ConnectionCommandAlreadyRegisteredException, DuplicatedCommandException, IllegalArgumentException {
        assert connectionCommand != null;

        if (this.connectionCommand == null) {
            this.registersCommand(connectionCommand);
            this.connectionCommand = connectionCommand;
        } else {
            throw new ConnectionCommandAlreadyRegisteredException(connectionCommand.getName());
        }
    }

    @Override
    public void registersDisconnectionCommand(final DisconnectionCommand disconnectionCommand)
            throws DisconnectionCommandAlreadyRegisteredException, DuplicatedCommandException,
            IllegalArgumentException {
        assert disconnectionCommand != null;

        if (this.disconnectionCommand == null) {
            this.registersCommand(disconnectionCommand);
            this.disconnectionCommand = disconnectionCommand;
        } else {
            throw new DisconnectionCommandAlreadyRegisteredException(disconnectionCommand.getName());
        }
    }

    @Override
    public Preferences getPreferences() {
        return this.preferences;
    }

    @Override
    public PrintStream getPrintStream() {
        return this.printStream;
    }

    @Override
    public PrintStream getErrorStream() {
        return this.errorStream;
    }

    /**
     * @return The content of the shell output stream as {@link String}
     */
    public String getOutput() {
        return this.baosOutputStream.toString();
    }

    /**
     * @return The content of the shell error stream as {@link String}
     */
    public String getError() {
        return this.baosErrorStream.toString();
    }

    @Override
    public void onConnectionEstablished() {
        this.isConnectionEstablished = true;
    }

    @Override
    public void onDisconnection() {
        this.isConnectionEstablished = false;
    }

    @Override
    public boolean isConnectionEstablished() {
        return this.isConnectionEstablished;
    }
}
